//
// HYPEROID.H - hyperoid internal header information
//
// Version: 1.1  Copyright (C) 1990,91 Hutchins Software
//      This software is licenced under the GNU General Public Licence
//      Please read the associated legal documentation
// Author: Edward Hutchins
// Revisions:
//
//	rasmus ekman - 96-09-13: Win32-port etc.
//


#ifndef RC_INVOKED

#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include <limits.h>
// Not used, since the Win95 close button isn't there (???) - re
///#define OEMRESOURCE
#include "winext.h"

//
// typedefs and defines
//

// color stuff
#define PALETTE_SIZE 16
typedef enum
{
	BLACK, DKGREY, GREY, WHITE,
	DKRED, RED, DKGREEN, GREEN, DKBLUE, BLUE,
	DKYELLOW, YELLOW, DKCYAN, CYAN, DKMAGENTA, MAGENTA
} COLORS;

// degrees scaled to integer math
#define DEGREE_SIZE 256
#define DEGREE_MASK 255
#define DEGREE_MAX 0x4000

// object limits
#define MAX_PTS 14			// Allow larger roids (orig: 8)  - re 96-09-18
#define MAX_OBJS 2000			// Allow more objects (orig: 100) 
// New constants to make field size variable - re 96-09-18
///#define MAX_COORD 0x2000		// ---> variable nFieldSize
///#define CLIP_COORD (MAX_COORD+300)	// ---> variable nClipCoord
#define MAX_FIELD 0x5000	// Max field size NB! Affects MAX_PTS
#define DEF_FIELD 0x3000	// Default field size
#define MIN_FIELD 0x1000	// Min field size
#define COORD_STEP 0x400	// Change between field size steps

// timer stuff
#define DRAW_TIMER 1
#define DRAW_DELAY 50
#define RESTART_TIMER 2
#define RESTART_DELAY 4000	// Slightly quicker (orig 5000) - re 96-09-13
#define SETLEVEL_TIMER 3	// For new level (F1-F12 el +/-) - re 96-09-24
#define SHORTER_DELAY 2500		// Even quicker, am I too impatient?

// restart modes (RESTART_NEWSIZE is new - re 96-09-18)
typedef enum {
	RESTART_GAME, RESTART_LEVEL, RESTART_NEXTLEVEL, 
	RESTART_CHOSENLEVEL, RESTART_NEWSIZE 
} RESTART_MODE;

// letter scaling
#define LETTER_MAX 256

// extra life every
#define EXTRA_LIFE 100000

// list node
typedef struct tagNODE
{
	struct tagNODE  *npNext, *npPrev;
} NODE;
pointerdef( NODE );

// list header
typedef struct
{
	NPNODE          npHead, npTail;
} LIST;

pointerdef( LIST );

// object descriptor
typedef struct
{
	NODE            Link;               // for object list
	POINT           Pos;                // position of center of object
	POINT           Vel;                // velocity in logical units/update
	INT             nMass;              // mass of object
	INT             nDir;               // direction in degrees
	INT             nSpin;              // angular momentum degrees/update
	INT             nCount;             // used by different objects
	INT             nDelay;             // used by different objects
	BYTE            byColor;            // palette color
	BYTE            byPts;              // number of points in object
	POINT           Pts[MAX_PTS];       // points making up an object
	POINT           Old[MAX_PTS];       // last plotted location
} OBJ;
pointerdef( OBJ );

//
// inline macro functions
//

// function aliases
#define AddHeadObj(l,o) AddHead((l),((NPNODE)o))
#define RemHeadObj(l) ((NPOBJ)RemHead(l))
#define RemoveObj(l,o) Remove((l),((NPNODE)o))
#define HeadObj(l) ((NPOBJ)((l)->npHead))
#define NextObj(o) ((NPOBJ)((o)->Link.npNext))

// real-time check of the keyboard
#define IsKeyDown(x) (GetAsyncKeyState(x)<0)

// I HATE typing this allatime!
#define INTRES(x) MAKEINTRESOURCE(x)

// size of an array
#define DIM(x) (sizeof(x)/sizeof((x)[0]))

// faster than MulDiv!
#define MULDEG(x,y) ((INT)(((LONG)(x)*(y))/DEGREE_MAX))

// DEG - convert an integer into a degree lookup index
#define DEG(x) ((WORD)(x)&DEGREE_MASK)

// ACCEL - accelerate an object in a given direction
#define ACCEL(o,d,s) \
(((o)->Vel.x += MULDEG((s),nCos[DEG(d)])), \
((o)->Vel.y += MULDEG((s),nSin[DEG(d)])))

// PTINRECT - a faster PtInRect
#define PTINRECT(r,p) \
(((r)->left <= (p).x) && ((r)->right > (p).x) && \
((r)->top <= (p).y) && ((r)->bottom > (p).y))

// INTRECT - a faster IntersectRect that just returns the condition
#define INTRECT(r1,r2) \
(((r1)->right >= (r2)->left) && \
((r1)->left < (r2)->right) && \
((r1)->bottom >= (r2)->top) && \
((r1)->top < (r2)->bottom))

// MKRECT - make a rect around a point
#define MKRECT(r,p,s) \
(((r)->left = ((p).x-(s))), ((r)->right = ((p).x+(s))), \
((r)->top = ((p).y-(s))), ((r)->bottom = ((p).y+(s))))

//
// prototypes
//

// hyperoid.c 
// All FAR/NEAR PASCAL ---> WINAPI, also removed EXPORTs - re
INT WINAPI arand( INT x );
VOID WINAPI AddHead( NPLIST npList, NPNODE npNode );
NPNODE WINAPI RemHead( NPLIST npList );
VOID WINAPI Remove( NPLIST npList, NPNODE npNode );
VOID WINAPI DrawObject( HDC hDC, NPOBJ npObj );
VOID WINAPI SetRestart( int bGameOver );
VOID WINAPI AddExtraLife( VOID );
VOID WINAPI Hit( HDC hDC, NPOBJ npObj );
VOID WINAPI Explode( HDC hDC, NPOBJ npObj );
BOOL WINAPI HitPlayer( HDC hDC, NPOBJ npObj );
NPOBJ WINAPI CreateLetter( CHAR cLetter, INT nSize );
VOID WINAPI DrawLetters( HDC hDC );
VOID WINAPI DrawHunterShots( HDC hDC );
VOID WINAPI FireHunterShot( NPOBJ npHunt );
VOID WINAPI CreateHunter( VOID );
VOID WINAPI DrawHunters( HDC hDC );
VOID WINAPI CreateSpinner( VOID );
VOID WINAPI DrawSpinners( HDC hDC );
VOID WINAPI CreateRoid( POINT Pos, POINT Vel, INT nSides, BYTE byColor, INT nDir, INT nSpeed, INT nSpin );
VOID WINAPI BreakRoid( HDC hDC, NPOBJ npRoid, NPOBJ npShot );
VOID WINAPI DrawRoids( HDC hDC );
VOID WINAPI DrawShots( HDC hDC );
VOID WINAPI DrawFlames( HDC hDC );
VOID WINAPI FireShot( VOID );
VOID WINAPI AccelPlayer( INT nDir, INT nAccel );
VOID WINAPI DrawPlayer( HDC hDC );
VOID WINAPI DrawObjects( HWND hWnd );
VOID WINAPI CheckScore( HWND hWnd );
VOID WINAPI HitList( HDC hDC, NPLIST npList );
VOID WINAPI ExplodeBadguys( HDC hDC, NPLIST npList );
// Redeclared, to allow different types of restart
// for F1-F12 and +/-, n/N - re 96-09-18
VOID WINAPI NewGame( HWND hWnd, INT type );
VOID WINAPI RestartHyperoid( VOID );
VOID WINAPI Panic( BOOL bPanic );
VOID WINAPI PaintHyperoid( HWND hWnd );
// Added a missing prototype - re 96-09-18
BOOL WINAPI EraseHyperoidBkgnd( HWND hWnd, HDC hDC );
VOID WINAPI DisableHyperoidInput( HWND hWnd, BOOL bCapture );
LONG WINAPI HyperoidWndProc( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam );
BOOL WINAPI InitHyperoid( VOID );
VOID WINAPI ExitHyperoid( VOID );

// roidsupp.c
VOID WINAPI PrintLetters( NPSTR npszText, POINT Pos, POINT Vel, BYTE byColor, INT nSize );
VOID WINAPI SpinLetters( NPSTR npszText, POINT Pos, POINT Vel, BYTE byColor, INT nSize );
HPALETTE WINAPI CreateHyperoidPalette( VOID );
BOOL WINAPI CreateHyperoidClass( VOID );
VOID WINAPI SetHyperoidMenu( HWND hWnd, INT nFirstID, INT nLastID );
HWND WINAPI CreateHyperoidWindow( LPSTR lpszCmd, INT nCmdShow );
VOID WINAPI SaveHyperoidWindowPos( HWND hWnd );
VOID WINAPI GetHyperoidIni( VOID );
VOID WINAPI HyperoidHelp( HWND hWnd );
BOOL WINAPI HyperoidAboutDlg( HWND hDlg, UINT mess, WPARAM wParam, LPARAM lParam );
VOID WINAPI AboutHyperoid( HWND hWnd );

#endif // RC_INVOKED //

//
// resource IDs
//

// icons and bitmaps
#define IDI_HYPEROID    10
#define IDI_PANIC       20

// bitmaps
#define IDB_BLANK       50
#define IDB_BOMB        51
#define IDB_LEVEL       52
#define IDB_LIFE        53
#define IDB_NUM0        54
#define IDB_NUM1        55
#define IDB_NUM2        56
#define IDB_NUM3        57
#define IDB_NUM4        58
#define IDB_NUM5        59
#define IDB_NUM6        60
#define IDB_NUM7        61
#define IDB_NUM8        62
#define IDB_NUM9        63
#define IDB_PLUS        64

#define IDB_SCORE       65
#define IDB_SHIELD      66
// Nya ikoner fr att visa fltstorl i titelraden (kanske) - re 96-09-20
#define IDB_SIZE        67
#define IDB_MINUS       68
#define IDB_NUMPlus     69
// Icon bmp fr titelraden + Win32 buttons - re 96-09-13
#define IDB_HYPERICO    80
#define IDB_RESTORE     90
#define IDB_ZOOM        91
// additional bitmap stuff
#define IDB_MAX         20    // Number of bitmaps IDB_BLANK...numPlus (frut: shield)
#define CX_BITMAP       16
#define CY_BITMAP       16

// strings
#define IDS_NAME        100

// menus
#define IDM_NEW         200
#define IDM_ABOUT       201

// about box
#define IDD_ABOUT       500
#define IDD_A_HELP      501
#define IDD_A_HISCORE   502

