//
// ROIDSUPP - hyperoid support functions
//
// Version: 1.1  Copyright (C) 1991, Hutchins Software
//      This software is licenced under the GNU General Public Licence
//      Please read the associated legal documentation
// Author: Edward Hutchins
// Revisions:
// 11/01/91 added GNU General Public License - Ed.
//
// Win32 port and INI/Registry conversion - rasmus ekman 96-09-13
//

#include "Hyperoid.h"

//
// defines
//

// you may ask, "why did he embed all these string constants instead of
// using the resource file?". Good question. The answer is: I feel better
// knowing this stuff is part of the executable, and not part of the resource
// file (which can be changed by sneaky people). Or maybe I wuz lazy.
// If you don't like it, then YOU can change it!

// No, i don't mind, but anyone can still change it with a hex editor,
//    or indeed right here in the source. - re

#define NL "\x0d\x0a"

#define HYPEROID_HELP \
"The following keys control your ship:" NL NL \
"    Left, Right Arrow .... spin left or right" NL \
"    Down, Up Arrow ..... forward or reverse thrust" NL \
"    Space Bar .............. fire!" NL \
"    Tab ........................ shields" NL \
"    S ............................ smartbomb" NL NL \
"The following keys control the game:" NL NL \
"    Esc ......................... pause/boss key" NL \
"    F1-F12 .................... restart game at level 1-12" NL \
"    -,+ ........................... shrink or expand all objects" NL \
"    n, N ........................ restore object size to normal" NL NL \
"Note: You have 3 lives, unlimited fuel and firepower, 3 shields and 3 " \
"smartbombs. Your ship gets darker when you lose a life, but you keep on " \
"playing (unless you hit an asteroid). You get an extra life every 100,000 " \
"points. When you lose the game, you start over immediately and can finish " \
"off the current level (which should now be 0) before starting over at " \
"level 1 (There is no waiting around between games)." NL NL \
"New features in version 1.95: You can choose a level to play (1-12). " \
"You can also choose the size of the playing field, either compressing it, " \
"making all objects smaller, or expand it, bringing everything up close."

/*
#define HYPEROID_HELP2 \
"The HYPEROID.INI file can be created/modified to change default settings " \
"in Hyperoid. Here are some of the items you can set:" NL \
NL "[Hyperoid]" NL "Max=<0/1>" NL "{X,Y,W,H}=<n>" NL "Mono=<0/1>" NL \
"DrawDelay=<ms> ;microseconds/frame" NL \
"FieldSize=<sz> ; -8 (very small) < 0 (normal) < 4 (very large)" NL NL\
"[Palette]" NL \
"{Black,DkGrey,Grey,White," NL \
" DkRed,Red,DkGreen,Green,DkBlue,Blue," NL \
" DkYellow,Yellow,DkCyan,Cyan," NL \
" DkMagenta,Magenta}=<r>,<g>,<b>" NL \
NL "[Keys]" NL \
"{Shield,Clockwise,CtrClockwise," NL \
" Thrust,RevThrust,Fire,Bomb}=<virtual keycode>" NL NL \
"Note: Virtual keycodes usually match the key's ASCII value."
*/

// Modified to reflect Registry init handling - re 96-09-23
#define HYPEROID_HELP2 \
"The Register keys for Hyperoid can be modified to change the default settings. " NL \
"The basic key is found at HKEY_LOCAL_MACHINE\\SOFTWARE\\Games\\Hyperoid\\. " NL NL \
"Here are some of the items you can set:" NL \
"     FieldSize = <sz> ; -8 (very small) < 0 (normal) < 8 (very large)" NL \
"     Maximized = <0/1>" NL \
"     {WindowX,WindowY,WindowW,WindowH} = <n>" NL \
"     DrawDelay = <ms> ;microseconds/frame" NL NL \
"Under the subkey Palette, you may set the following values:" NL \
"     Mono = <0/1>" NL \
"     {Black,DkGrey,Grey,White," NL \
"      DkRed,Red,DkGreen,Green,DkBlue,Blue," NL \
"      DkYellow,Yellow,DkCyan,Cyan," NL \
"      DkMagenta,Magenta} = <r,g,b>" NL NL \
"Under the subkey Keys, you may set the following values:" NL \
"     {Shield,Clockwise,CtrClockwise," NL \
"      Thrust,RevThrust,Fire,Bomb} = <virtual keycode>" NL NL \
"Virtual keycodes usually match the key's ASCII value." NL NL \
"IMPORTANT: All keys should be created as string values."


#define HYPEROID_HELPSTYLE (MB_OK | MB_ICONASTERISK)

// this is the part I especially want in the executable image

#define HYPEROID_LICENSE \
"This program is free software; you can redistribute it and/or modify " \
"it under the terms of the GNU General Public License as published by " \
"the Free Software Foundation; either version 1, or (at your option) " \
"any later version. " \
NL NL \
"This program is distributed in the hope that it will be useful, " \
"but WITHOUT ANY WARRANTY; without even the implied warranty of " \
"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the " \
"GNU General Public License for more details. " \
NL NL \
"You should have received a copy of the GNU General Public License " \
"along with this program; if not, write to the Free Software " \
"Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. "

//
// imports
//

IMPORT CHAR         szAppName[32] FROM( hyperoid.c );
IMPORT HINSTANCE    hAppInst FROM( hyperoid.c );		// HANDLE
IMPORT BOOL         bBW FROM( hyperoid.c );
IMPORT INT          nDrawDelay FROM( hyperoid.c );
IMPORT INT          vkShld FROM( hyperoid.c );
IMPORT INT          vkClkw FROM( hyperoid.c );
IMPORT INT          vkCtrClkw FROM( hyperoid.c );
IMPORT INT          vkThrst FROM( hyperoid.c );
IMPORT INT          vkRvThrst FROM( hyperoid.c );
IMPORT INT          vkFire FROM( hyperoid.c );
IMPORT INT          vkBomb FROM( hyperoid.c );
IMPORT LONG         lHighScore FROM( hyperoid.c );
// New, for the field size - re 96-09-18
IMPORT INT          nFieldSize FROM( hyperoid.c );
IMPORT INT          wClipCoord FROM( hyperoid.c );
IMPORT INT          hClipCoord FROM(hyperoid.c);

//
// globals
//

// these parts map to "abcdefghijklm"
// Ie, maps to points in a LED-style grid:
//   a b c
//    d e
//   f g h
//    i j
//   k l m     - re
GLOBAL POINT        LetterPart[] =
{
	{83, 572}, {64, 512}, {45, 572}, {96, 362}, {32, 362},
	{128, 256}, {0, 0}, {0, 256},
	{160, 362}, {224, 362}, {173, 572}, {192, 512}, {211, 572}
};
// here's the vector font
GLOBAL NPSTR        szNumberDesc[] =
{
	"cakmck",       // 0
	"dbl",          // 1
	"abekm",        // 2
	"abegjlk",      // 3
	"mcfh",         // 4
	"cbfgjlk",      // 5
	"bdiljgi",      // 6
	"acgl",         // 7
	"bdjlieb",      // 8
	"ljebdge"       // 9
};
GLOBAL NPSTR        szLetterDesc[] =
{
	"kdbemhf",      // A
	"kabegjlk",     // B
	"cbflm",        // C
	"kabejlk",      // D
	"cafgfkm",      // E
	"cafgfk",       // F
	"bdiljhg",      // G
	"kafhcm",       // H
	"bl",           // I
	"cjli",         // J
	"akcgm",        // K
	"akm",          // L
	"kagcm",        // M
	"kamc",         // N
	"bdiljeb",      // O
	"kabegf",       // P
	"mlidbejl",     // Q
	"kabegfgm",     // R
	"ebdjli",       // S
	"lbac",         // T
	"ailjc",        // U
	"alc",          // V
	"akgmc",        // W
	"amgkc",        // X
	"aglgc",        // Y
	"ackm"          // Z
};

//
// locals
//

LOCAL CHAR          szIni[] = "HYPEROID.INI";
LOCAL CHAR          szLicense[] = "LicenseRead";
LOCAL CHAR          szDrawDelay[] = "DrawDelay";
LOCAL CHAR          szMax[] = "Maximized";
LOCAL CHAR          szX[] = "WindowX";
LOCAL CHAR          szY[] = "WindowY";
LOCAL CHAR          szW[] = "WindowW";
LOCAL CHAR          szH[] = "WindowH";
LOCAL CHAR          szBW[] = "Mono";
LOCAL CHAR          szPalette[] = "Palette";
LOCAL CHAR          szKeys[] = "Keys";
LOCAL CHAR          szShield[] = "Shield";
LOCAL CHAR          szClockwise[] = "Clockwise";
LOCAL CHAR          szCtrClockwise[] = "CtrClockwise";
LOCAL CHAR          szThrust[] = "Thrust";
LOCAL CHAR          szRevThrust[] = "RevThrust";
LOCAL CHAR          szFire[] = "Fire";
LOCAL CHAR          szBomb[] = "Bomb";
LOCAL CHAR          szHi[] = "HiScore";
// New for field size handling - re 96-09-18
LOCAL CHAR          szFieldSize[] = "FieldSize";
// New for registry handling - re 96-09-20
LOCAL CHAR          szRegKey[] = "SOFTWARE\\Games\\Hyperoid\\";

LOCAL CHAR          *szColorName[] =
{
	"Black", "DkGrey", "Grey", "White",
	"DkRed", "Red",  "DkGreen", "Green", "DkBlue", "Blue",
	"DkYellow", "Yellow", "DkCyan", "Cyan", "DkMagenta", "Magenta"
};
LOCAL DWORD         dwColors[] =
{
	RGB(0,0,0), RGB(128,128,128),
	RGB(192,192,192), RGB(255,255,255),
	RGB(128,0,0), RGB(255,0,0),
	RGB(0,128,0), RGB(0,255,0),
	RGB(0,0,128), RGB(0,0,255),
	RGB(128,128,0), RGB(255,255,0),
	RGB(0,128,128), RGB(0,255,255),
	RGB(128,0,128), RGB(255,0,255),
};

//
// PrintLetters - create letter objects from a string
//

VOID WINAPI PrintLetters( NPSTR npszText, POINT Pos, POINT Vel,
							  BYTE byColor, INT nSize )
{
	INT             nLen = strlen( npszText );
	INT             nCnt = nLen;
	INT             nSpace = nSize + nSize / 2;
	INT             nBase = (nLen - 1) * nSpace;
	INT             nBaseStart = Pos.x + nBase / 2;

	while (nCnt--)
	{
		NPOBJ npLtr = CreateLetter( npszText[nCnt], nSize / 2 );
		if (npLtr)
		{
			npLtr->Pos.x = nBaseStart;
			npLtr->Pos.y = Pos.y;
			npLtr->Vel = Vel;
			npLtr->byColor = byColor;
		}
		nBaseStart -= nSpace;
	}
}

//
// SpinLetters - spin letter objects away from center for effect
//

VOID WINAPI SpinLetters( NPSTR npszText, POINT Pos, POINT Vel,
							 BYTE byColor, INT nSize )
{
	INT             nLen = strlen( npszText );
	INT             nCnt = nLen;
	INT             nSpace = nSize + nSize / 2;
	INT             nBase = (nLen - 1) * nSpace;
	INT             nBaseStart = Pos.x + nBase / 2;

	while (nCnt--)
	{
		NPOBJ npLtr = CreateLetter( npszText[nCnt], nSize / 2 );
		if (npLtr)
		{
			INT nSpin = (nCnt - nLen / 2) * 2;
			npLtr->Pos.x = nBaseStart;
			npLtr->Pos.y = Pos.y;
			npLtr->Vel = Vel;
			npLtr->Vel.x += nSpin * 16;
			npLtr->nSpin = -nSpin;
			npLtr->byColor = byColor;
		}
		nBaseStart -= nSpace;
	}
}

//
// CreateHyperoidPalette - create a logical palette
// Modernized control of color abilities - re 96-09-19
// Changed to work with the Registry - re 96-09-23

HPALETTE WINAPI CreateHyperoidPalette( VOID )
{
	HPALETTE        hPalette;
	HDC             hIC = CreateIC( "DISPLAY", NULL, NULL, NULL );
	INT             t = 0, nColors;
	PALETTEENTRY    Pal[PALETTE_SIZE + 2];
	NPLOGPALETTE    npLogPalette = (NPLOGPALETTE)Pal;
	HKEY            hKey = 0;
	DWORD            lValue = 0, lResult;
	DWORD           dwType;
	CHAR            szBuff[32], szKeyName[MAX_PATH];	// For palette key - re

	// Earlier version assumed max 8 bit colour depth
	nColors = GetDeviceCaps( hIC, NUMCOLORS );
	// Units with capability exceeding 8 bits return 0xFFFFFFFF
	if(nColors == 0xFFFFFFFF || nColors >= 8) bBW = FALSE;
	else bBW = TRUE;		// are we forced into using b&w?
	DeleteDC( hIC );
	// Open or create subkey for palette
	wsprintf(szKeyName, "%s%s\\", szRegKey, szPalette);
	lResult = RegCreateKeyEx(HKEY_LOCAL_MACHINE, szKeyName, 0, szAppName, 0,
		KEY_QUERY_VALUE, NULL, &hKey, &lValue);
	// Check against registry if user manually set B/W 
	szBuff[0] = 0;
	lValue = sizeof(szBuff);
	lResult = RegQueryValueEx(hKey, szBW, 0, &dwType, (BYTE*)&szBuff, &lValue);
	if(szBuff[0] == '1' && lResult == ERROR_SUCCESS) bBW = TRUE;

	npLogPalette->palVersion = 0x0300;
	npLogPalette->palNumEntries = PALETTE_SIZE;

	for (t = 0; t < PALETTE_SIZE; ++t)
	{
		DWORD           dwColor = dwColors[t];

		// Must be reset every turn
		lValue = sizeof(szBuff);	
		szBuff[0] = '\0';
		// Check for a value before getting it
		lResult = RegQueryValueEx(hKey, szColorName[t], 0, &dwType, (BYTE*)&szBuff, &lValue);
		if (szBuff[0] && lResult == ERROR_SUCCESS)
		{
			INT r, g, b;
			NPSTR npBuff = szBuff;
			r = g = b = 255;
			while (*npBuff == ' ') ++npBuff;
			r = atoi( npBuff );
			while (*npBuff && *npBuff != ',') ++npBuff;
			if (*npBuff == ',') g = atoi( ++npBuff );
			while (*npBuff && *npBuff != ',') ++npBuff;
			if (*npBuff == ',') b = atoi( ++npBuff );
			dwColor = RGB( r, g, b );
		}
		if (bBW) dwColor = ((dwColor == RGB(0,0,0)) ? RGB(0,0,0) : RGB(255,255,255));
		npLogPalette->palPalEntry[t].peRed = GetRValue( dwColor );
		npLogPalette->palPalEntry[t].peGreen = GetGValue( dwColor );
		npLogPalette->palPalEntry[t].peBlue = GetBValue( dwColor );
		npLogPalette->palPalEntry[t].peFlags = 0;
	}

	hPalette = CreatePalette( npLogPalette );
	return( hPalette );
}

//
// CreateHyperoidClass - create the class of Hyperoid's window
// Fixed to work with the WNDCLASSEX structure - re 97-01-01

BOOL WINAPI CreateHyperoidClass( VOID )
{
	WNDCLASSEX        Class;

	// load the name from the resource file
	LoadString( hAppInst, IDS_NAME, szAppName, sizeof(szAppName) );

	Class.cbSize = sizeof(WNDCLASSEX);
	Class.style = CS_HREDRAW | CS_VREDRAW;
	Class.lpfnWndProc = HyperoidWndProc;
	Class.cbClsExtra = 0;
	Class.cbWndExtra = 0;
	Class.hInstance = hAppInst;
	Class.hIcon = LoadIcon( hAppInst, INTRES(IDI_HYPEROID) );
	Class.hCursor = LoadCursor( NULL, IDC_CROSS );
	Class.hbrBackground = HNULL;
	Class.lpszMenuName = szAppName;
	Class.lpszClassName = szAppName;
	Class.hIconSm = LoadIcon( hAppInst, INTRES(IDI_HYPEROID) );


	return( RegisterClassEx( &Class ) );
}

//
// SetHyperoidMenu - add Hyperoid's menu items to the system menu
//

VOID WINAPI SetHyperoidMenu( HWND hWnd, INT nFirstID, INT nLastID )
{
	CHAR            szMenuName[40];
	HMENU           hMenu;

	hMenu = GetSystemMenu( hWnd, TRUE );
	if (hMenu == HNULL) hMenu = GetSystemMenu( hWnd, FALSE );
	if (hMenu == HNULL) return;

	while (nFirstID <= nLastID)
	{
		LoadString( hAppInst, nFirstID, szMenuName, sizeof(szMenuName) );
		ChangeMenu( hMenu, 0, szMenuName, nFirstID, MF_APPEND );
		++nFirstID;
	}
}


//
// CreateHyperoidWindow - open the Hyperoid window
// Fixed to work with Registry - re

HWND WINAPI CreateHyperoidWindow( LPSTR lpszCmd, INT nCmdShow )
{
	HWND            hWnd;
	HKEY            hKey = 0;
	LONG            lResult;
	DWORD           lValue, dwType, nLic;
	INT             x, y, w, h;		// nLic is new - re
	CHAR            szBuff[32];

	// Create the key
	lResult = RegCreateKeyEx(HKEY_LOCAL_MACHINE, szRegKey, 0, szAppName, 0,
		KEY_QUERY_VALUE || KEY_WRITE, NULL, &hKey, &lValue);

	// get the highscore profile here for lack of a better place...
	lValue = sizeof(DWORD);
	lResult = RegQueryValueEx(hKey, szHi, 0, &dwType, (BYTE*)&lHighScore, &lValue);
	if(lHighScore <= 0 || lResult != ERROR_SUCCESS) lHighScore = 0;

	// x
	lValue = sizeof(szBuff); szBuff[0] = '\0';
	lResult = RegQueryValueEx(hKey, szX, 0, &dwType, (BYTE*)&szBuff, &lValue);
	if (szBuff[0] == '\0' || lResult != ERROR_SUCCESS) x = CW_USEDEFAULT;
	else x = atol( szBuff );
	// y
	lValue = sizeof(szBuff);	szBuff[0] = '\0';
	lResult = RegQueryValueEx(hKey, szY, 0, &dwType, (BYTE*)&szBuff, &lValue);
	if(szBuff[0] == '\0' || lResult != ERROR_SUCCESS) y = CW_USEDEFAULT;
	else y = atol( szBuff );
	// w
	lValue = sizeof(szBuff);	szBuff[0] = '\0';
	lResult = RegQueryValueEx(hKey, szW, 0, &dwType, (BYTE*)&szBuff, &lValue);
	if(szBuff[0] == '\0' || lResult != ERROR_SUCCESS) w = CW_USEDEFAULT;
	else w = atol( szBuff );
	// h
	lValue = sizeof(szBuff);	szBuff[0] = '\0';
	lResult = RegQueryValueEx(hKey, szH, 0, &dwType, (BYTE*)&szBuff, &lValue);
	if(szBuff[0] == '\0' || lResult != ERROR_SUCCESS) h = CW_USEDEFAULT;
	else h = atol( szBuff );

	// nCmdShow
	lValue = sizeof(szBuff);	szBuff[0] = '\0';
	lResult = RegQueryValueEx(hKey, szMax, 0, &dwType, (BYTE*)&szBuff, &lValue);
	if(szBuff[0] == '1' && lResult == ERROR_SUCCESS) nCmdShow = SW_SHOWMAXIMIZED;
	else nCmdShow = SW_NORMAL;

	// license
	lValue = sizeof(INT);
	nLic = 0;
	lResult = RegQueryValueEx(hKey, szLicense, 0, &dwType, (BYTE*)&nLic, &lValue);

	// CreateWindow
	hWnd = CreateWindowEx( WS_EX_CLIENTEDGE, szAppName, szAppName, 
		WS_OVERLAPPEDWINDOW, x, y, w, h, HNULL, HNULL, hAppInst, NULL );
	if (hWnd == HNULL) return( HNULL );

	ShowWindow( hWnd, nCmdShow );
	UpdateWindow( hWnd );
	SetHyperoidMenu( hWnd, IDM_NEW, IDM_ABOUT );

	// show the license...
	if(nLic != 1 || lResult != ERROR_SUCCESS) {
		MessageBeep( HYPEROID_HELPSTYLE );
		MessageBox( hWnd, HYPEROID_LICENSE, "Hyperoid License", HYPEROID_HELPSTYLE );
		// ...and never show it again (unless they want to see it)
		nLic = 1;
		RegSetValueEx(hKey, szLicense, 0, REG_DWORD, (BYTE*)&nLic, sizeof(DWORD));
	}

	RegCloseKey(hKey);

	return( hWnd );
}

//
// SaveHyperoidWindowPos - write out the .ini information
// Fixed to work with Registry - re

VOID WINAPI SaveHyperoidWindowPos( HWND hWnd )
{
	HKEY        hKey = 0;
	LONG        lResult;
	DWORD       lValue;
	RECT        rect;
	CHAR        szBuff[32];

	// Create the key
	lResult = RegCreateKeyEx(HKEY_LOCAL_MACHINE, szRegKey, 0, szAppName, 0,
		KEY_WRITE, NULL, &hKey, &lValue);
	if(lResult != ERROR_SUCCESS) return;

	// save the highscore profile here for lack of a better place...
	if (lHighScore)
	{
		RegSetValueEx(hKey, szHi, 0, REG_DWORD, (BYTE*)&lHighScore, sizeof(LONG));
	}

	// Set licence-read again since it doesn't work in win 8
	lValue = 1;
	RegSetValueEx(hKey, szLicense, 0, REG_DWORD, (BYTE*)&lValue, sizeof(DWORD));

	szBuff[0] = '\0';
	// Save field size in format (-8 < 0 < 8; as shown on screen) - re 96-09-18
	wsprintf(szBuff, "%d",  (DEF_FIELD - nFieldSize) / COORD_STEP);
	RegSetValueEx(hKey, szFieldSize, 0, REG_SZ, (BYTE*)&szBuff, sizeof(szBuff));

	if (IsIconic( hWnd )) return;
	if (IsZoomed( hWnd ))
	{
		RegSetValueEx(hKey, szMax, 0, REG_SZ, (BYTE*)"1", 2 * sizeof(CHAR));
		return;
	} else {
		RegSetValueEx(hKey, szMax, 0, REG_SZ, (BYTE*)&"0", 2 * sizeof(CHAR));
	}

	GetWindowRect( hWnd, &rect );
	wsprintf( szBuff, "%d", rect.left );
	RegSetValueEx(hKey, szX, 0, REG_SZ, (BYTE*)&szBuff, sizeof(szBuff));
	wsprintf( szBuff, "%d", rect.top );
	RegSetValueEx(hKey, szY, 0, REG_SZ, (BYTE*)&szBuff, sizeof(szBuff));
	wsprintf( szBuff, "%d", rect.right - rect.left );
	RegSetValueEx(hKey, szW, 0, REG_SZ, (BYTE*)&szBuff, sizeof(szBuff));
	wsprintf( szBuff, "%d", rect.bottom - rect.top );
	RegSetValueEx(hKey, szH, 0, REG_SZ, (BYTE*)&szBuff, sizeof(szBuff));

	RegCloseKey(hKey);
}

//
// GetHyperoidIni - load the ini file information
// Fixed to work with Registry - re 96-09-23

VOID WINAPI GetHyperoidIni( VOID )
{
	HKEY        hKey = 0;
	LONG        lResult;
	DWORD       lValue;
	DWORD       dwType;
	CHAR        szBuff[32], szKeyName[MAX_PATH];

	// Create the key
	lResult = RegCreateKeyEx(HKEY_LOCAL_MACHINE, szRegKey, 0, szAppName, 0,
		KEY_QUERY_VALUE, NULL, &hKey, &lValue);

	szBuff[0] = '0';
	lValue = sizeof(szBuff);
	lResult = RegQueryValueEx(hKey, szDrawDelay, NULL, &dwType, (BYTE*)&szBuff, &lValue);
	if(szBuff[0] && lResult == ERROR_SUCCESS) nDrawDelay = atol(szBuff);
	else nDrawDelay = DRAW_DELAY;

	// New stuff to get field size - re 96-09-18
	lValue = sizeof(szBuff);	szBuff[0] = '\0';
	lResult = RegQueryValueEx(hKey, szFieldSize, NULL, &dwType, (BYTE*)&szBuff, &lValue);
	if(szBuff[0] == '\0' || lResult != ERROR_SUCCESS) nFieldSize = DEF_FIELD;
	// Convert from relative size to the virtual coordinate systeme
	else nFieldSize = -atol(szBuff) * COORD_STEP + DEF_FIELD;
	if(nFieldSize < MIN_FIELD) nFieldSize = MIN_FIELD;
	else if(nFieldSize > MAX_FIELD) nFieldSize = MAX_FIELD;

	// Create/open the subkey "Keys"
	wsprintf(szKeyName, "%s%s\\", szRegKey, szKeys);
	lResult = RegCreateKeyEx(HKEY_LOCAL_MACHINE, szKeyName, 0, szAppName, 0,
		KEY_QUERY_VALUE, NULL, &hKey, &lValue);
	// vkShld
	lValue = sizeof(szBuff);	szBuff[0] = '\0';
	lResult = RegQueryValueEx(hKey, szShield, NULL, &dwType, (BYTE*)&szBuff, &lValue);
	if(szBuff[0] == '\0' || lResult != ERROR_SUCCESS) vkShld = VK_TAB;
	else vkShld = atol( szBuff );
	// vkClkw
	lValue = sizeof(szBuff);	szBuff[0] = '\0';
	lResult = RegQueryValueEx(hKey, szClockwise, NULL, &dwType, (BYTE*)&szBuff, &lValue);
	if(szBuff[0] == '0' || lResult != ERROR_SUCCESS) vkClkw = VK_LEFT;
	else vkClkw = atol( szBuff );
	// vkCtrClkw
	lValue = sizeof(szBuff);	szBuff[0] = '\0';
	lResult = RegQueryValueEx(hKey, szCtrClockwise, NULL, &dwType, (BYTE*)&szBuff, &lValue);
	if(szBuff[0] == '\0' || lResult != ERROR_SUCCESS) vkCtrClkw = VK_RIGHT;
	else vkCtrClkw = atol( szBuff );
	// vkThrst
	lValue = sizeof(szBuff);	szBuff[0] = '\0';
	lResult = RegQueryValueEx(hKey, szThrust, NULL, &dwType, (BYTE*)&szBuff, &lValue);
	if(szBuff[0] == '0' || lResult != ERROR_SUCCESS) vkThrst = VK_DOWN;
	else vkThrst = atol( szBuff );
	// vkRvThrst
	lValue = sizeof(szBuff);	szBuff[0] = '\0';
	lResult = RegQueryValueEx(hKey, szRevThrust, NULL, &dwType, (BYTE*)&szBuff, &lValue);
	if(szBuff[0] == '\0' || lResult != ERROR_SUCCESS) vkRvThrst = VK_UP;
	else vkRvThrst = atol( szBuff );
	// vkFire
	lValue = sizeof(szBuff);	szBuff[0] = '\0';
	lResult = RegQueryValueEx(hKey, szFire, NULL, &dwType, (BYTE*)&szBuff, &lValue);
	if(szBuff[0] == '\0' || lResult != ERROR_SUCCESS) vkFire = VK_SPACE;
	else vkFire = atol( szBuff );
	// vkBomb
	lValue = sizeof(szBuff);	szBuff[0] = '\0';
	lResult = RegQueryValueEx(hKey, szBomb, NULL, &dwType, (BYTE*)&szBuff, &lValue);
	if(szBuff[0] == '\0' || lResult != ERROR_SUCCESS) vkBomb = 'S';
	else vkBomb = atol( szBuff );

	RegCloseKey(hKey);
}

//
// HyperoidHelp - show help
//

VOID WINAPI HyperoidHelp( HWND hWnd )
{
	MessageBox( hWnd, HYPEROID_HELP, "Hyperoid help", HYPEROID_HELPSTYLE );
	MessageBox( hWnd, HYPEROID_HELP2, "Hyperoid.ini help", HYPEROID_HELPSTYLE );
}

//
// HyperoidAboutDlg - the about box proc
// The usual porting changes, also removed "EXPORT" from declaration - re 96-09-13

BOOL WINAPI HyperoidAboutDlg( HWND hDlg, UINT mess,
										 WPARAM wParam, LPARAM lParam )
{
	switch (mess)
	{
	case WM_INITDIALOG:
		if (lHighScore)
		{
			CHAR szBuff[40];
			wsprintf( szBuff, "High Score: %7.7lu", lHighScore );
			SetDlgItemText( hDlg, IDD_A_HISCORE, szBuff );
		}
		break;

	case WM_COMMAND:
		switch (wParam)
		{
		case IDD_A_HELP:
			HyperoidHelp( hDlg );
			// fall through...
		case IDOK:
			EndDialog( hDlg, 0 );
			break;

		default:
			return( FALSE );
		}
		break;

	case WM_CLOSE:
		EndDialog( hDlg, FALSE );
		break;

	default:
		return( FALSE );
	}
	return( TRUE );
}

//
// AboutHyperoid - show the about box
//

VOID WINAPI AboutHyperoid( HWND hWnd )
{
	DialogBox( hAppInst, INTRES( IDD_ABOUT ), hWnd, HyperoidAboutDlg );
}
